#include "POOLCore/Context.h"
#include "POOLCore/IBlobStreamingService.h"
#include "POOLCore/Exception.h"
#include "RelationalAccess/ISessionProxy.h"

#include "RelationalAccess/ConnectionService.h"

std::string
pool::Context::automaticSchemaEvolutionFlagName(){
  static std::string s_automaticSchemaEvolutionFlagName("POOL_ORA_AUTOMATIC_SCHEMA_EVOLUTION");
  return s_automaticSchemaEvolutionFlagName;
}

coral::ISessionProxy*
pool::Context::getSession( const std::string& connectionString,
                           coral::AccessMode accessMode,
                           std::map<std::string,coral::ISessionProxy*>& sessionPool){
  coral::ISessionProxy* session = 0;
  std::map<std::string,coral::ISessionProxy*>::const_iterator iS=sessionPool.find( connectionString );
  if( iS != sessionPool.end()){
    session = iS->second;
  } else {
    session = m_connectionService->connect( connectionString, accessMode );
    sessionPool.insert(std::make_pair( connectionString, session ));
  }
  return session;
}

pool::Context::Context():m_blobStreamingService(0),m_blobStreamingServiceOwnership(false),
                         m_connectionService(0),m_connectionServiceOwnership(false),
                         m_readOnlySessions(),m_updateSessions(),m_sessionSharing(false){
}

pool::Context::~Context(){
  if(m_blobStreamingService && m_blobStreamingServiceOwnership){
    delete m_blobStreamingService;
  }  
  if(m_connectionService && m_connectionServiceOwnership){
    // to be fixed!
    delete dynamic_cast<const coral::ConnectionService*>(m_connectionService);
    //delete m_connectionService;
  }
  if(m_sessionSharing){
    for(std::map<std::string,coral::ISessionProxy*>::const_iterator iS=m_readOnlySessions.begin();
        iS != m_readOnlySessions.end(); ++iS){
      delete iS->second;
    }
    for(std::map<std::string,coral::ISessionProxy*>::const_iterator iS=m_updateSessions.begin();
        iS != m_updateSessions.end(); ++iS){
      delete iS->second;
    }
  }
}

void
pool::Context::setBlobStreamer(pool::IBlobStreamingService* service, bool giveUpOwnership){
  if(m_blobStreamingService && m_blobStreamingServiceOwnership){
    delete m_blobStreamingService;
  }
  m_blobStreamingService = service;
  m_blobStreamingServiceOwnership = giveUpOwnership;
}

void 
pool::Context::setConnectionService(coral::IConnectionService* service, bool giveUpOwnership){
  if(m_connectionService && m_connectionServiceOwnership){
    // to be fixed!
    delete dynamic_cast<const coral::ConnectionService*>(m_connectionService);
    //delete m_connectionService;
  }
  m_connectionService = service;
  m_connectionServiceOwnership = giveUpOwnership;
  m_connectionService = service;
}

coral::ISessionProxy&
pool::Context::sharedSession( const std::string& connectionString, coral::AccessMode accessMode ){
  if(!m_connectionService){
    throw pool::Exception("Connection service is not installed.","pool::Context::sharedSession","POOLCore");
  }
  if(!m_sessionSharing){
    throw pool::Exception("Connection Sharing is not enabled.","pool::Context::sharedSession","POOLCore");
  }
  coral::ISessionProxy* session = 0;
  if(accessMode == coral::ReadOnly){
    session = getSession( connectionString, accessMode, m_readOnlySessions );
  } else {
    session = getSession( connectionString, accessMode, m_updateSessions );
  }
  return *session;
}

