#include "POOLCore/PContainerBase.h"
#include "POOLCore/Selection.h"
#include "POOLCore/Exception.h"

pool::RefCounter::RefCounter():m_ref(0){
}

pool::RefCounter::~RefCounter(){
}

int pool::RefCounter::release(){
  m_ref--;
  return m_ref;
}

int pool::RefCounter::addRef(){
  m_ref++;
  return m_ref;
}

pool::IContainerLoader::IContainerLoader():RefCounter(){
}

void pool::PContainerBase::releaseData(pool::IContainerData* dataReference){
  if(dataReference){
    int refs = dataReference->release();
    if(refs<1) delete dataReference;
  }
}

void pool::PContainerBase::except(const std::string& message) const{
  throw pool::Exception(message, std::string(""), std::string(""));
}

pool::LoaderClient::LoaderClient():m_loader(0){
}

pool::LoaderClient::LoaderClient(IContainerLoader& loader):m_loader(&loader){
  m_loader->addRef();
}

pool::LoaderClient::LoaderClient( const LoaderClient& rhs ):m_loader(rhs.m_loader){
  if(m_loader) m_loader->addRef();
}

pool::LoaderClient& pool::LoaderClient::operator=(const pool::LoaderClient& rhs ){
  if(&rhs != this){
    pool::IContainerLoader* prevLoader = m_loader;
    m_loader= rhs.m_loader;
    if(m_loader) m_loader->addRef();
    if(prevLoader) {
      int refs = prevLoader->release();
      if(refs<1) {
        prevLoader->notify();
        delete prevLoader;
      }
    }
  }
  return *this;
}

pool::LoaderClient::~LoaderClient(){
  if(m_loader){
    int refs = m_loader->release();
    if(refs<1) {
      m_loader->notify();
      delete m_loader;
    }
  }
}

pool::IContainerLoader& pool::LoaderClient::loader() const{
  if(!m_loader) throw pool::Exception("Loader has not been installed.",std::string(""), std::string(""));
  return *m_loader;
}

bool pool::LoaderClient::hasLoader() const {
  return m_loader!=0;
}

void pool::LoaderClient::install(IContainerLoader& loader){
  if(m_loader != &loader){
    if(m_loader){
      int refs = m_loader->release();
      if(refs<1) {
        m_loader->notify();
        delete m_loader;
      }
    }
    m_loader = &loader;
    m_loader->addRef();
  }
}



