#include "DataElement.h"

pool::RelationalStorageService::DataElement::DataElement():
  m_parent(0),
  m_children(),
  m_declaringScopeOffset(0),
  m_offsetFunction(0),
  m_metadataIndex(-1),
  m_isRef(false),
  m_hasMetadata( false){
}

pool::RelationalStorageService::DataElement::DataElement( size_t declaringScopeOffset,
                                                          Reflex::OffsetFunction offsetFunc ):
  m_parent(0),
  m_children(),
  m_declaringScopeOffset(declaringScopeOffset),
  m_offsetFunction(offsetFunc),
  m_metadataIndex(-1),
  m_isRef(false),
  m_hasMetadata( false){
}

pool::RelationalStorageService::DataElement::~DataElement(){
  for(std::vector<DataElement*>::const_iterator iEl = m_children.begin();
      iEl != m_children.end(); ++iEl ){
    delete *iEl;
  }
}

pool::RelationalStorageService::DataElement&
pool::RelationalStorageService::DataElement::addChild( size_t declaringScopeOffset,
                                                       Reflex::OffsetFunction offsetFunction ){
  DataElement* child = new DataElement( declaringScopeOffset, offsetFunction );
  child->m_parent = this;
  m_children.push_back(child);
  return *child;
}

size_t pool::RelationalStorageService::DataElement::offset( const void* topLevelAddress ) const {
  const void* address = topLevelAddress;
  size_t offset = m_declaringScopeOffset;
  if(m_parent){
    size_t parentOffset = m_parent->offset( topLevelAddress );
    offset += parentOffset;
    address = static_cast<char*>(const_cast<void*>(topLevelAddress))+parentOffset;
  }
  if(m_offsetFunction){
    offset += m_offsetFunction( const_cast<void*>(address));
  }
  return offset;  
}

void* pool::RelationalStorageService::DataElement::address( const void* topLevelAddress ) const {
  void* elementAddress = static_cast< char* >( const_cast<void*>(topLevelAddress)) + offset( topLevelAddress );
  return elementAddress;
}

size_t pool::RelationalStorageService::DataElement::declaringScopeOffset() const 
{
  return m_declaringScopeOffset;
}

void 
pool::RelationalStorageService::DataElement::setRefFlag(){
  m_isRef = true;
}

void 
pool::RelationalStorageService::DataElement::setMetadata( size_t metadataIndex ){
  m_metadataIndex = metadataIndex;
  m_hasMetadata = true;
}

bool
pool::RelationalStorageService::DataElement::isRef() const {
  return m_isRef;
}

bool
pool::RelationalStorageService::DataElement::hasMetadata() const {
  return m_hasMetadata;
}

size_t pool::RelationalStorageService::DataElement::metadataIndex() const {
  return m_metadataIndex;
}
