#ifndef RELATIONALSTORAGESERVICE_OBJECTREADER_H
#define RELATIONALSTORAGESERVICE_OBJECTREADER_H

#include "DbIOHandler.h"
#include "Reflex/Type.h"
#include <vector>
#include <set>

#include "RVersion.h"

#if ROOT_VERSION_CODE < ROOT_VERSION(5,19,0)
using namespace ROOT;
#endif

namespace coral {

  class ISchema;
  class IQuery;
  class ICursor;
  class AttributeList;
}

namespace pool {

  class ObjectRelationalMappingElement;
  class DataCallBack;
  class Selection;

  namespace RelationalStorageService {

    class ObjectRelationalReadDataBuffer;
    class MappingManager;
    class RelationalPtrLoader;
    class RelationalContainerLoader;

    class ObjectReader : virtual public DbIOHandler {
    public:
      /// Constructor
      ObjectReader( coral::ISchema& schema,
                    const Reflex::Type& dictionary,
                    MappingManager& mappingManager,
                    pool::DbDatabase* databaseHandle );

      /// Destructor
      ~ObjectReader();

      /// Reads an object
      bool readObject( void* objectData,
                       int oid,
                       bool anyObject,
                       pool::DataCallBack& callback );

      void* loadPoolPointer( ObjectRelationalReadDataBuffer& buffer,
                             int oid,
                             bool anyObject,
                             pool::DataCallBack& callback );
      
      bool loadPoolContainer( const std::vector<int>& identity,
                              const Selection& selection,
                              ObjectRelationalReadDataBuffer& buffer,
                              void* address,
                              pool::DataCallBack& callback );

      size_t makeCountQuery( const std::vector<int>& identity,
                             const Selection& selection,
                             ObjectRelationalReadDataBuffer& buffer );

      bool eraseLoader( const RelationalPtrLoader* loader );
      bool eraseLoader( const RelationalContainerLoader* loader );

      private:
      
      bool readObjectData( coral::ICursor& queryData,
                           ObjectRelationalReadDataBuffer& buffer,
                           void* objectData,
                           int& oid,
                           pool::DataCallBack& callback );

      /// Reads an array
      bool readArray( const std::vector<int>& identity,
                      ObjectRelationalReadDataBuffer& buffer,
                      void* address,
                      pool::DataCallBack& callback );

      /// Reads an array
      bool readPoolContainer( const std::vector<int>& identity,
                              ObjectRelationalReadDataBuffer& buffer,
                              void* address,
                              pool::DataCallBack& callback );

      /// Reads a pool ptr
      bool readPoolPointer( ObjectRelationalReadDataBuffer& buffer,
                            void* objectData,
                            int oid,
                            bool anyObject,
                            pool::DataCallBack& callback );
      
      /// Reads a pool polymorphic ptr
      bool readPoolPolymorphicPointer( ObjectRelationalReadDataBuffer& buffer,
                                       void* objectData,
                                       int oid,
                                       bool anyObject,
                                       pool::DataCallBack& callback );
    private:

      coral::IQuery* makeQuery( ObjectRelationalReadDataBuffer& buffer,
                                int oid,
                                bool anyObject);


      bool processDependentBuffers( ObjectRelationalReadDataBuffer& buffer,
                                    void* objectData,
                                    int oid,
                                    bool anyObject,
                                    pool::DataCallBack& callback );

      std::string setQueryCondition(const pool::Selection& selection, const ObjectRelationalMappingElement& mappingElement, coral::AttributeList& conditionData);
      
      private:
      /// The relational schema
      coral::ISchema& m_schema;
      /// The map of mapping elements to element entries
      ObjectRelationalReadDataBuffer* m_dataBuffer;
      /// The dictionary for a pool reference
      const Reflex::Type m_refDict;
      /// The container mapping manager
      MappingManager& m_mappingManager;
      /// The pointer loaders
      std::set<RelationalPtrLoader*> m_poolPointerLoaders;
      /// The container loaders
      std::set<RelationalContainerLoader*> m_poolContainerLoaders;
    };

  }

}

#endif
